use crate::prelude::*;

use std::ptr;

#[repr(C)]
#[derive(Debug)]
pub struct VkSubpassDescription {
    pub flags: VkSubpassDescriptionFlagBits,
    pub pipelineBindPoint: VkPipelineBindPoint,
    pub inputAttachmentCount: u32,
    pub pInputAttachments: *const VkAttachmentReference,
    pub colorAttachmentCount: u32,
    pub pColorAttachments: *const VkAttachmentReference,
    pub pResolveAttachments: *const VkAttachmentReference,
    pub pDepthStencilAttachment: *const VkAttachmentReference,
    pub preserveAttachmentCount: u32,
    pub pPreserveAttachments: *const u32,
}

impl VkSubpassDescription {
    pub fn new<T>(
        flags: T,
        input_attachments: &[VkAttachmentReference],
        color_attachments: &[VkAttachmentReference],
        resolve_attachments: &[VkAttachmentReference],
        depth_stencil_attachment: Option<&VkAttachmentReference>,
        preserve_attachments: &[u32],
    ) -> Self
    where
        T: Into<VkSubpassDescriptionFlagBits>,
    {
        VkSubpassDescription {
            flags: flags.into(),
            // the only bit currently supported
            pipelineBindPoint: VK_PIPELINE_BIND_POINT_GRAPHICS,
            inputAttachmentCount: input_attachments.len() as u32,
            pInputAttachments: input_attachments.as_ptr(),
            colorAttachmentCount: color_attachments.len() as u32,
            pColorAttachments: color_attachments.as_ptr(),
            pResolveAttachments: if resolve_attachments.is_empty() {
                ptr::null()
            } else {
                debug_assert!(
                    resolve_attachments.len() == color_attachments.len(),
                    "there must be as much resolve attachements as color attachments, if used"
                );
                resolve_attachments.as_ptr()
            },
            pDepthStencilAttachment: match depth_stencil_attachment {
                Some(attachment) => attachment as *const _,
                None => ptr::null(),
            },
            preserveAttachmentCount: preserve_attachments.len() as u32,
            pPreserveAttachments: preserve_attachments.as_ptr(),
        }
    }
}
